import { world, system, EntityEquippableComponent, EquipmentSlot, ItemStack } from "@minecraft/server";
import { shields_loop } from "./shield";
import { damageItem } from "./lib/durability";
const w = world.getDimension("overworld");
system.runInterval(() =>{
	try{
		for (const p of world.getPlayers()) {
			item_animations(p);
			shields_loop(p)
			const trident = w.getEntities({type: `hydra:spectre_t`})
			for(const e of trident){
				if(e.hasTag("sensor")){
						e.getTags().forEach(t=>{
							if(t.startsWith('{"itemTag":{')){
								const itemId = JSON.parse(t)['itemTag']['id']
								const itemDag = JSON.parse(t)['itemTag']['damage']
								const drop_item = e.dimension.spawnEntity("new:trident_equip", {x: e.location.x, y: e.location.y, z: e.location.z})
								drop_item.runCommandAsync(`replaceitem entity @s slot.weapon.mainhand 0 ${itemId} 1 ${itemDag}`)
								e.runCommandAsync("event entity @s new:item_spawn")
								e.removeTag(t)
								system.runTimeout(()=>{
									drop_item.runCommandAsync("event entity @s drop_equip")
								}, 1)
							}
						}
					)
				}
			}
		}
	}catch(e){}
})
world.afterEvents.itemStopUse.subscribe(use =>{
	const item = use.itemStack;
	const p = use.source;
	if(item.typeId.includes("hydra:spectre_trident") && p.hasTag("use_trident")){
		custom_trident(p, item)
	}
})
world.afterEvents.itemUse.subscribe(use =>{
	const item = use.itemStack;
	const p = use.source;
	const cooldown=item.getComponent("cooldown")
	if(item.typeId.includes("hydra:spectre_trident") && !p.hasTag("use_trident")){
		p.addTag("use_trident")
	}

	if(item.typeId=="hydra:spectre_cane"&&!cooldown.getCooldownTicksRemaining(p)){
		const entity = p.dimension.spawnEntity("hydra:fire_ball", p.getHeadLocation());
		const projectile = entity.getComponent('projectile');
		projectile.owner = p;
		projectile.shoot({x: p.getViewDirection().x*2, y: p.getViewDirection().y*2, z: p.getViewDirection().z*2});
		cooldown.startCooldown(p)
		Durability(item, p)
		p.playSound("random.wand")
		p.dimension.spawnParticle("minecraft:totem_particle", p.location)
		p.playAnimation("animation.player.spectre_wand",{
			stopExpression: "v.is_first_person"
		})
		
	}
})
export function Durability(tool, source) {
    try{
        system.run(()=>{
            const component = source.getComponent(EntityEquippableComponent.componentId);
            const durability = tool.getComponent("durability");
            const enchantable = tool.getComponent("enchantable");
            if(tool && durability !== undefined){
                let unbreakingLevel = 0;
                const damageAmount = 2;
                if (enchantable) {
                    for (const enchant of enchantable.getEnchantments()) {
                        if (enchant.type.id === "unbreaking") {
                            unbreakingLevel = enchant.level;
                            break;
                        }
                    }
                }if(durability.damage+2 < durability.maxDurability){
                    if(unbreakingLevel > 0) {
                        const chance = 1 / (unbreakingLevel + 1);
                        if (Math.random() < chance) {
                            durability.damage += damageAmount;
                        }
                    } if(unbreakingLevel == 0) {
                        durability.damage += damageAmount;
                    }
                    component.setEquipment(EquipmentSlot.Mainhand, tool)
                }else if(durability.damage+2 >= durability.maxDurability){
                    const air = new ItemStack("minecraft:air")
					component.setEquipment(EquipmentSlot.Mainhand, air)
                    source.playSound("random.break")
                }
            }
        })
    }catch(e){}
}

export function custom_trident(p, item){
	const damage = item.getComponent("durability").damage;
	const trident_item = item.typeId;
	system.runTimeout(()=>{
		const equip = p.getComponent(EntityEquippableComponent.componentId);
		const hand = equip?.getEquipment(EquipmentSlot.Mainhand);
		if(hand){
			if(hand.typeId == trident_item){
				p.removeTag("use_trident")
				const tridentItem = item.typeId.replace("rident", "")
				const trident = p.dimension.spawnEntity(`${tridentItem}`, p.getHeadLocation());
				const projectile = trident.getComponent('projectile');
				const itemTag = {
					'itemTag': {
						'id': `${item.typeId}`,
						'damage': `${damage+2}`
					}
				};
				trident.addTag(JSON.stringify(itemTag))
				projectile.owner = p;
				projectile.shoot({x: p.getViewDirection().x*4, y: p.getViewDirection().y*4, z: p.getViewDirection().z*4});
				p.runCommandAsync(`replaceitem entity @s slot.weapon.mainhand 0 air`)
				p.runCommandAsync("playsound item.trident.throw @s")
			}
			if(hand.typeId != trident_item){
				if(p.hasTag("use_trident")){
					p.removeTag("use_trident")
				}
			}
		}
		if(hand == undefined){
			if(p.hasTag("use_trident")){
				p.removeTag("use_trident")
			}
		}
	}, 1)
}



export function item_animations(p) {
	const equip = p.getComponent(EntityEquippableComponent.componentId);
	try{
		const item = equip?.getEquipment(EquipmentSlot.Mainhand);
		if(item.typeId.includes("hydra:spectre_trident") && p.hasTag("use_trident")){
			p.runCommandAsync(`playanimation @s animation.player.gem_trident root 100`)
			p.addTag("trident")
		}
		if(!item.typeId.includes("hydra:spectre_trident") || !p.hasTag("use_trident")){
			if(p.hasTag("trident")){
				p.runCommandAsync("playanimation @s animation.player.gem_trident root")
				p.removeTag("trident")
			}
		}
		if(item.hasTag("template:shield") && !p.hasTag("mainhand") && !p.hasTag("offhand") && p.isSneaking){
				p.runCommandAsync(`playanimation @s animation.player.custom_shield_block_main_hand root 100`)
				p.addTag("mainhand")
			}
		if(!item.hasTag("template:shield") || p.hasTag("offhand") || !p.isSneaking){
			if(p.hasTag("mainhand")){
				p.runCommandAsync("playanimation @s animation.player.custom_shield_block_main_hand root")
				p.removeTag("mainhand")
			}
		}
	}catch(e){
		if(p.hasTag("mainhand")){
			p.runCommandAsync("playanimation @s animation.player.custom_shield_block_main_hand root")
			p.removeTag("mainhand")
		}
		if(p.hasTag("trident")){
			p.runCommandAsync("playanimation @s animation.player.gem_trident root")
			p.removeTag("trident")
		}
	}
	try{
		let off = equip?.getEquipment(EquipmentSlot.Offhand);
		if(off){
			if(off.hasTag("template:shield")&&p.isSneaking){
				p.addTag("offhand")
				p.runCommandAsync(`playanimation @s animation.player.custom_shield_block_off_hand root 100`)
			}
			if(!off.hasTag("template:shield")||!p.isSneaking){
				if(p.hasTag("offhand")){
					p.runCommandAsync("playanimation @s animation.player.custom_shield_block_off_hand root")
					p.removeTag("offhand")
				}
			}
		}
		if(!off){
			if(p.hasTag("offhand")){
				p.runCommandAsync("playanimation @s animation.player.custom_shield_block_off_hand root")
				p.removeTag("offhand")
			}
		}
	}catch(e){
		
	}
}